function check_basics_motions_z(Mzs,Mzps,b_output)
%%  check basic relations for motions with partially exponential repr.
%
% twist vector of adjoint motion
% twist vector of inverse motion
% function for inverse motion
% twist vector of concatenated motion
% twist vector of relative motion
% difference motion M12 as concatenated of inv(M1) and M2
%
% wf 5/2020

% for generating small twist vectors
sigma_n = 10^-8;
% M1 and M
Mz = Mzs.Mz;
Mzp = Mzps.Mzp;

%% twist vector 
zeta      = randn(6,1)*sigma_n;
% adjoint motion of a small motion
M_zeta    = [expm(calc_S(zeta(1:3))) zeta(4:6); 0 0 0 1];
M_zeta_ad = Mz * M_zeta * inv(Mz);

%adjoint of motion matrix
M_ad = [Mz(1:3,1:3) zeros(3); calc_S(Mz(1:3,4))*Mz(1:3,1:3) Mz(1:3,1:3)];

% twist vector of adjoint motion
zeta_ad = calc_s_from_A(M_zeta_ad);

% check
check_zeta_ad = (zeta_ad - M_ad * zeta)';
if b_output
check_zeta_ad = (zeta_ad - M_ad * zeta)'
end

%% inverse

% noisy small motion matrix and its inverse
M_zeta_r    = [expm(calc_S(zeta(1:3)))*Mz(1:3,1:3) Mz(1:3,4)+zeta(4:6); 0 0 0 1];
M_zeta_inv  = inv(M_zeta_r);

% twist vector of inverse
zeta_inv = calc_z_from_M_M0(M_zeta_inv,inv(Mz));

% inverse of adjoint of motion matrix
M_ad_zeta_inv        = [Mz(1:3,1:3)' zeros(3); Mz(1:3,1:3)'*calc_S(Mz(1:3,4)) Mz(1:3,1:3)'];

% check
check_zeta_inv = (zeta_inv - (-M_ad_zeta_inv  * zeta))';
if b_output
check_zeta_inv = (zeta_inv - (-M_ad_zeta_inv  * zeta))'
end

% check forward and backward: M = (inv(inv(M))
[Mzsi,J] = calc_inverse_M_z(Mzs);
MM       = calc_inverse_M_z(Mzsi);

% check
check_Mz_inv = Mzs.Mz-MM.Mz;
check_Cz_inv = Mzs.Cz-MM.Cz;
if b_output
check_Mz_inv = Mzs.Mz-MM.Mz
check_Cz_inv = Mzs.Cz-MM.Cz
end

%% concatenation M = M2*M1

% twist vectors of noise of motion pair
zeta   = randn(12,1)*sigma_n;
% noisy motion pair
M1_r   = [expm(calc_S(zeta(1:3)))*Mzp(1:3,1:3) Mzp(1:3,4)+zeta(4:6); 0 0 0 1];
M2_r   = [expm(calc_S(zeta(7:9)))*Mzp(1:3,5:7) Mzp(1:3,8)+zeta(10:12); 0 0 0 1];
% true and noisy concatenation
M      = Mzp(:,5:8) * Mzp(:,1:4);
M_r    = M2_r * M1_r;
% twist vector of noise of concatenations
zeta_e = calc_z_from_M_M0(M_r,M);

% check zeta = [rho;tau]
R2     = Mzp(1:3,5:7);
Z1     = Mzp(1:3,4);

check_zeta_concatenate_rho  = ...
    zeta_e(1:3) - (R2 * zeta(1:3) + zeta(7:9)); 
check_zeta_concatenate_tau  = ...
    zeta_e(4:6) - (R2 * zeta(4:6) + zeta(10:12) - calc_S(R2*Z1)*zeta(7:9));

check_zeta_concatenate = [check_zeta_concatenate_rho;check_zeta_concatenate_tau]';
if b_output
check_zeta_concatenate = [check_zeta_concatenate_rho;check_zeta_concatenate_tau]'
end


%% relative motion M = inv(M1)*M2

% twist vectors of noise of motion pair
zeta    = randn(12,1)*sigma_n;
% noisy motion pair
M1_r   = [expm(calc_S(zeta(1:3)))*Mzp(1:3,1:3) Mzp(1:3,4)+zeta(4:6); 0 0 0 1];
M2_r   = [expm(calc_S(zeta(7:9)))*Mzp(1:3,5:7) Mzp(1:3,8)+zeta(10:12); 0 0 0 1];
% true and noisy relative motion
M    = inv(Mzp(:,1:4)) * Mzp(:,5:8) ;
M_r  = inv(M1_r)       * M2_r;
% twist vector of noise of relative motion
zeta_e = calc_z_from_M_M0(M_r,M);

% % adjunct motion matrix of M1
% R1     = Mzp(1:3,1:3);
% Z1     = Mzp(1:3,4);
% M_1_ad = [R1 zeros(3); calc_S(Z1)*R1 R1];

% check of zeta = [rho;tau]
R1     = Mzp(1:3,1:3);
Z1     = Mzp(1:3,4);
Z2     = Mzp(1:3,8);

check_zeta_concatenate_rho  = ...
    zeta_e(1:3) - (R1' * (zeta(7:9) - zeta(1:3)));
check_zeta_concatenate_tau  = ...
    zeta_e(4:6) - (R1' * (zeta(10:12) - zeta(4:6)) ...
    + R1' * calc_S(Z2-Z1) * zeta(1:3));

check_zeta_relative = [check_zeta_concatenate_rho;check_zeta_concatenate_tau]';
if b_output
check_zeta_relative = [check_zeta_concatenate_rho;check_zeta_concatenate_tau]'
end

%% check all three functions, assume independence
% motions M1 and M2
Mz1.Mz   = Mzps.Mzp(:,1:4);
Mz2.Mz   = Mzps.Mzp(:,5:8);
% CovM of M1, M2
C1       = Mzps.Czp(1:6,1:6);
C2       = Mzps.Czp(7:12,7:12);
% CovM of independent motion pair
C        = [C1 zeros(6); zeros(6) C2];
Mz1.Cz   = C1;
Mz2.Cz   = C2;
Mzps.Czp = C;

% relative motion Rd = R1' * R2
Mzd      = calc_relative_M_z(Mzps);
% check with concatenation: 
% inverse of M1
Mz1i     = calc_inverse_M_z(Mz1);
% concatenate 
Mz12.Mzp = [Mz2.Mz,Mz1i.Mz];
Mz12.Czp = [Mz2.Cz zeros(6); zeros(6) Mz1i.Cz];
Mzd_ch   = calc_concatenated_M_z(Mz12);

% check rel(M1,M2) = con(M2,inv(M1))
check_relative_Mz = Mzd_ch.Mz-Mzd.Mz;
check_relative_Cz = Mzd_ch.Cz-Mzd.Cz;
if b_output
check_relative_Mz = Mzd_ch.Mz-Mzd.Mz
check_relative_Cz = Mzd_ch.Cz-Mzd.Cz
end

% total evaluation
T = 10^-6;
if norm(check_zeta_ad)         < T*sigma_n && ...
        norm(check_zeta_inv)   < T*sigma_n && ...
        norm(check_Mz_inv)     < T*sigma_n && ...
        norm(check_Cz_inv)     < T*sigma_n^2 && ...
        norm(check_zeta_concatenate) < T*sigma_n && ...
        norm(check_zeta_relative)    < T*sigma_n && ...
        norm(check_relative_Mz)      < T*sigma_n && ...
        norm(check_relative_Cz)      < T*sigma_n^2
    
    display('motions zeta      ok +++++++++++++++++++++++++++++++++++++++')
else
    display('notions zeta      not ok ***********************************')
end
end

