function check_simulated_motion_z(M0,C0,Ms,S)
%%  check whether Ms has mean M0 and CovM C0
%   partially exponential representation
%
% M0    [3x3] mean motion zeta
% C0    [6x6] theoretical CovM if zeta
% Ms    [16xN] sample of motions zeta
% S     significance level
%
% wf 5/2020

% number of samples
N = size(Ms,2);

% analyse data
zeta_s=zeros(N,6);
for n=1:N
    zeta_s(n,:)= calc_z_from_M_M0(reshape(Ms(:,n),4,4),(M0));
end

% check mean and CovM
check_estimation_result(6, zeros(6,1), C0, ones(N,1), zeta_s, S, ' motions z');

CovM_z=cov(zeta_s);
sigmas_z=sqrt(diag(CovM_z)')

%% the same assuming the exponential representation
% analyse data
s_s=zeros(N,6);
for n=1:N
    s_s(n,:)= calc_s_from_A(reshape(Ms(:,n),4,4)*inv(M0));
end

% check mean and CovM
check_estimation_result(6, zeros(6,1), C0, ones(N,1), s_s, S, ' motions z/s');

check_estimation_result(3, zeros(3,1), C0(1:3,1:3), ones(N,1), s_s(:,1:3), S, ' rotations z/s');
check_estimation_result(3, zeros(3,1), C0(4:6,4:6), ones(N,1), s_s(:,4:6), S, ' translations z/s');


CovM_z_s=cov(s_s);

sigmas_z_s=sqrt(diag(CovM_z_s)')
end

